// Convert sol data to array (Original converted to XML)
// Ported from http://osflash.org/s2x (Public Domain, No Licencing issues.)
// by Trevor Hobson
// Added Pointer data type - TH
var objectionSolRead = {
	NUMBER: '\x00',
	BOOLEAN: '\x01',
	STRING: '\x02',
	OBJOBJ: '\x03',
	NULL: '\x05',
	UNDEF: '\x06',
	POINTER: '\x07',
	OBJARR: '\x08',
	//RAWARR: '\x0A',
	OBJDATE: '\x0B',
	OBJM: '\x0D',
	OBJXML: '\x0F',
	OBJCC: '\x10',
	FALSE3: '\x02',
	TRUE3: '\x03',
	NUMBER3: '\x04',
	STRING3: '\x06',
	contentArray: [],
	count: 0,
	pointerTarget: 1,

	read: function(solFile)
	{
		// Check to make sure a file was passed
		try
		{
			if (!solFile.isFile() || !solFile.exists())
			{
				return null;
			}
		} catch(er)
		{
			return null;
		}
		objectionSolRead.count = 0;
		objectionSolRead.pointerTarget = 1;
		objectionSolRead.contentArray = new Array();
		try
		{
			objectionSolRead.readSol(solFile);
			return objectionSolRead.contentArray;
		} catch(er)
		{
			return null;
		}
	},
	
	readSol: function(solFile)
	{
		// Open the file as a bitstream
		var istream = Components.classes["@mozilla.org/network/file-input-stream;1"].createInstance(Components.interfaces.nsIFileInputStream);
		istream.init(solFile, -1, -1, false);

		var bstream = Components.classes["@mozilla.org/binaryinputstream;1"].createInstance(Components.interfaces.nsIBinaryInputStream);
		bstream.setInputStream(istream);

		var nLenFile = bstream.available();
		var sHeader = bstream.readBytes(2);
		var nLenData = bstream.read32(4); // (Unsigned) Long
		if (nLenFile != nLenData + 6)
		{
			objectionSolRead.contentArray[objectionSolRead.count] = new objectionSolRead.contentFrame(objectionSolRead.count++, 'error', 'Data Length Mismatch', 'error', 0, false, 0);
			return;
		}
		var sFileType = bstream.readBytes(4); // 'TCSO'
		var sth1 = bstream.readBytes(6);
		var nLenSoln = bstream.read16(2); // (Unsigned) Short
		var solname = bstream.readBytes(nLenSoln);
		var nVersion = bstream.read32(4); // (Unsigned) Long
		objectionSolRead.contentArray[objectionSolRead.count] = new objectionSolRead.contentFrame(objectionSolRead.count++, 'name', solname, nVersion, treeLevel, false, 0);
		if (nVersion == 0)
		{
			var treeLevel = 0;
			try
			{
				while (bstream.available() > 0)
				{
					var nLenVname = bstream.read16(2); // (Unsigned) Short
					var sVname = bstream.readBytes(nLenVname);
					objectionSolRead.readStream(bstream, sVname, treeLevel);
					xEnd = bstream.readBytes(1); // '\x00'
				}
			} catch(er)
			{
				objectionSolRead.contentArray[objectionSolRead.count] = new objectionSolRead.contentFrame(objectionSolRead.count++, 'error', sVname, 'error', treeLevel, false, 0);
				bstream.close();
			}
		}
		// This bit is for the version 3 files. Not yet fully developed.
		/*
		else if (nVersion == 3)
		{
			var treeLevel = 0;
			try
			{
				while (bstream.available() > 0)
				{
					var sLenVname = bstream.readBytes(1); // Byte value determines length of name
					var nLenVname = Math.round((sLenVname.charCodeAt(0)/2)-.01);
					nLenVname = nLenVname.toFixed(0);
					var sVname = bstream.readBytes(nLenVname);
					objectionSolRead.v3readStream(bstream, sVname, treeLevel);
					xEnd = bstream.readBytes(1); // '\x00'
				}
			} catch(er)
			{
				objectionSolRead.contentArray[objectionSolRead.count] = new objectionSolRead.contentFrame(objectionSolRead.count++, 'error', sVname, 'error', treeLevel, false, 0);
				bstream.close();
			}
		}
		*/
		else
		{
			objectionSolRead.contentArray[objectionSolRead.count] = new objectionSolRead.contentFrame(objectionSolRead.count++, 'version', 'Not yet supported sol format', nVersion, treeLevel, false, 0);
		}
		bstream.close();
	},

	readStream: function(bstream, sVname, treeLevel)
	{
		var xType = bstream.readBytes(1);
		if (xType == objectionSolRead.NUMBER)
		{
			objectionSolRead.num2a(bstream, sVname, treeLevel);
		}
		else if (xType == objectionSolRead.BOOLEAN)
		{
			objectionSolRead.bol2a(bstream, sVname, treeLevel);
		}
		else if (xType == objectionSolRead.STRING)
		{
			objectionSolRead.str2a(bstream, sVname, treeLevel);
		}
		else if (xType == objectionSolRead.OBJOBJ)
		{
			objectionSolRead.obj2a(bstream, sVname, treeLevel);
		}
		else if (xType == objectionSolRead.OBJARR)
		{
			objectionSolRead.arr2a(bstream, sVname, treeLevel);
		}
		else if (xType == objectionSolRead.OBJDATE)
		{
			objectionSolRead.dat2a(bstream, sVname, treeLevel);
		}
		else if (xType == objectionSolRead.OBJXML)
		{
			objectionSolRead.xml2a(bstream, sVname, treeLevel);
		}
		else if (xType == objectionSolRead.OBJCC)
		{
			objectionSolRead.occ2a(bstream, sVname, treeLevel);
		}
		else if (xType == objectionSolRead.OBJM)
		{
			objectionSolRead.ojm2a(bstream, sVname, treeLevel);
		}
		else if (xType == objectionSolRead.POINTER)
		{
			objectionSolRead.ptr2a(bstream, sVname, treeLevel);
		}
		else if (xType == objectionSolRead.NULL)
		{
			objectionSolRead.nul2a(bstream, sVname, treeLevel);
		}
		else if (xType == objectionSolRead.UNDEF)
		{
			objectionSolRead.und2a(bstream, sVname, treeLevel);
		}
		else
		{
			var xTypeHex = xType.charCodeAt(0).toString(16)
			objectionSolRead.contentArray[objectionSolRead.count] = new objectionSolRead.contentFrame(objectionSolRead.count++, 'error', sVname, xTypeHex, treeLevel, false, 0);
		}
	},

	contentFrame: function(number, type, name, value, level, container, target)
	{
		this.number = number;
		this.type = type;
		this.name = name;
		this.value = value;
		this.level = level;
		this.container = container;
		this.target = target;
		return this;
	},

	num2a: function(bstream, sVname, treeLevel)
	{
		try
		{
			var s = bstream.readDouble(8); // Double
			var sValue = String(s);
		} catch(er)
		{
			var sValue = 'error';
		}
		objectionSolRead.contentArray[objectionSolRead.count] = new objectionSolRead.contentFrame(objectionSolRead.count++, 'number', sVname, sValue, treeLevel, false, 0);
	},

	bol2a: function(bstream, sVname, treeLevel)
	{
		var b = bstream.readBytes(1)
		if (b == '\x00')
		{
			var sValue = 'false';
		}
		else if (b == '\x01')
		{
			var sValue = 'true';
		}
		else
		{
			var sValue = 'error';
		}
		objectionSolRead.contentArray[objectionSolRead.count] = new objectionSolRead.contentFrame(objectionSolRead.count++, 'boolean', sVname, sValue, treeLevel, false, 0);
	},

	str2a: function(bstream, sVname, treeLevel)
	{
		var nLenStr = bstream.read16(2); // (Unsigned) Short
		var sValue = bstream.readBytes(nLenStr);
		objectionSolRead.contentArray[objectionSolRead.count] = new objectionSolRead.contentFrame(objectionSolRead.count++, 'string', sVname, sValue, treeLevel, false, 0);
	},

	obj2a: function(bstream, sVname, treeLevel)
	{
		objectionSolRead.contentArray[objectionSolRead.count] = new objectionSolRead.contentFrame(objectionSolRead.count++, 'object', sVname, '', treeLevel, true, objectionSolRead.pointerTarget++);
		var nLenVname = bstream.read16(2); // Unsigned Short
		while (nLenVname != 0)
		{
			var sVname = bstream.readBytes(nLenVname);
			objectionSolRead.readStream(bstream, sVname, treeLevel + 1);
			var nLenVname = bstream.read16(2); // (Unsigned) Short
		}
		xEndObj = bstream.readBytes(1); // '\x09'
	},

	arr2a: function(bstream, sVname, treeLevel)
	{
		var nArrLength = bstream.read32(4); // (Unsigned) Long
		objectionSolRead.contentArray[objectionSolRead.count] = new objectionSolRead.contentFrame(objectionSolRead.count++, 'array', sVname, '', treeLevel, true, objectionSolRead.pointerTarget++);
		var nLenVname = bstream.read16(2); // (Unsigned) Short
		while (nLenVname != 0)
		{
			var sVname = bstream.readBytes(nLenVname);
			objectionSolRead.readStream(bstream, sVname, treeLevel + 1);
			var nLenVname = bstream.read16(2); // (Unsigned) Short
		}
		xEndObj = bstream.readBytes(1); // '\x09'
	},

	dat2a: function(bstream, sVname, treeLevel)
	{
		var nMsec = bstream.readDouble(8); // Double
		var nMinOffset = bstream.read16(2); // (Signed) Short - Does not read negative as short so need to convert it
		if (nMinOffset > 0x7fff)
		{
			nMinOffset = ~(0x10000 - nMinOffset - 1);
		}
		var nOffset = nMinOffset/60;
		var sOffsetSign = (nOffset <= 0) ? '+' : ''; // Create a + for positive offset
		var sOffset = sOffsetSign + String(-nOffset);
		var nDate = new Date(Date.UTC(1969,12,1) + nMsec); // Create a date starting at Dec 1 1969
		var sDate = nDate.getUTCFullYear() + '-' + (nDate.getUTCMonth()+1) + '-' + nDate.getUTCDate() + ' ' + nDate.getUTCHours() + ':' + nDate.getUTCMinutes() + ':' + nDate.getUTCSeconds() + '.' + nDate.getUTCMilliseconds() + ' UTC ' + sOffset;
		objectionSolRead.contentArray[objectionSolRead.count] = new objectionSolRead.contentFrame(objectionSolRead.count++, 'date', sVname, sDate, treeLevel, false, 0);
	},

	xml2a: function(bstream, sVname, treeLevel)
	{
		var nLenCData = bstream.read32(4); // (Unsigned) Long
		var sCData = bstream.readBytes(nLenCData);
		objectionSolRead.contentArray[objectionSolRead.count] = new objectionSolRead.contentFrame(objectionSolRead.count++, 'xml', sVname, sCData, treeLevel, false, 0);
	},

	occ2a: function(bstream, sVname, treeLevel)
	{
		var nLenCname = bstream.read16(2); // (Unsigned) Short
		var sCname = bstream.readBytes(nLenCname);
		objectionSolRead.contentArray[objectionSolRead.count] = new objectionSolRead.contentFrame(objectionSolRead.count++, 'c_object', sVname, sCname, treeLevel, true, objectionSolRead.pointerTarget++);
		var nLenVname = bstream.read16(2); // (Unsigned) Short
		while (nLenVname != 0)
		{
			var sVname = bstream.readBytes(nLenVname);
			objectionSolRead.readStream(bstream, sVname, treeLevel + 1);
			var nLenVname = bstream.read16(2); // (Unsigned) Short
		}
		xEndObj = bstream.readBytes(1); // '\x09'
	},

	ojm2a: function(bstream, sVname, treeLevel)
	{
		objectionSolRead.contentArray[objectionSolRead.count] = new objectionSolRead.contentFrame(objectionSolRead.count++, 'm_object', sVname, '', treeLevel, false, objectionSolRead.pointerTarget++);
	},

	ptr2a: function(bstream, sVname, treeLevel)
	{
		var nPointer = bstream.read16(2); // (Unsigned) Short
		var sPointer = String(nPointer);
		objectionSolRead.contentArray[objectionSolRead.count] = new objectionSolRead.contentFrame(objectionSolRead.count++, 'pointer', sVname, sPointer, treeLevel, false, 0);
	},

	nul2a: function(bstream, sVname, treeLevel)
	{
		objectionSolRead.contentArray[objectionSolRead.count] = new objectionSolRead.contentFrame(objectionSolRead.count++, 'null', sVname, '', treeLevel, false, 0);
	},

	und2a: function(bstream, sVname, treeLevel)
	{
		objectionSolRead.contentArray[objectionSolRead.count] = new objectionSolRead.contentFrame(objectionSolRead.count++, 'undefined', sVname, '', treeLevel, false, 0);
	},
	
	v3readStream: function(bstream, sVname, treeLevel)
	{
		var xType = bstream.readBytes(1);
		if (xType == objectionSolRead.FALSE3)
		{
			objectionSolRead.v3bol2a('false', sVname, treeLevel);
		}
		else if (xType == objectionSolRead.TRUE3)
		{
			objectionSolRead.v3bol2a('true', sVname, treeLevel);
		}
		if (xType == objectionSolRead.NUMBER3)
		{
			objectionSolRead.v3num2a(bstream, sVname, treeLevel);
		}
		else if (xType == objectionSolRead.STRING3)
		{
			objectionSolRead.v3str2a(bstream, sVname, treeLevel);
		}
		else
		{
			var xTypeHex = xType.charCodeAt(0).toString(16)
			objectionSolRead.contentArray[objectionSolRead.count] = new objectionSolRead.contentFrame(objectionSolRead.count++, 'error', sVname, xTypeHex, treeLevel, false, 0);
		}
	},
	
	v3bol2a: function(sValue, sVname, treeLevel)
	{
		objectionSolRead.contentArray[objectionSolRead.count] = new objectionSolRead.contentFrame(objectionSolRead.count++, 'boolean', sVname, sValue, treeLevel, false, 0);
	},
	
	v3num2a: function(bstream, sVname, treeLevel)
	{
		var sPart = bstream.readBytes(1);
		var nPart = sPart.charCodeAt(0);
		var sValue = objectionSolRead.padHex(nPart.toString(16));
		while (nPart > 128)
		{
			sPart = bstream.readBytes(1);
			nPart = sPart.charCodeAt(0);
			sValue += objectionSolRead.padHex(nPart.toString(16));
		}
		objectionSolRead.contentArray[objectionSolRead.count] = new objectionSolRead.contentFrame(objectionSolRead.count++, 'number', sVname, sValue, treeLevel, false, 0);
	},
	
	v3str2a: function(bstream, sVname, treeLevel)
	{
		var sLenStr = bstream.readBytes(1); // Byte value determines length of string
		var nLenStr = Math.round((sLenStr.charCodeAt(0)/2)-.01);
		var sValue = bstream.readBytes(nLenStr);
		objectionSolRead.contentArray[objectionSolRead.count] = new objectionSolRead.contentFrame(objectionSolRead.count++, 'string', sVname, sValue, treeLevel, false, 0);
	},
	
	padHex: function(stringHex)
	{
		return (stringHex.length > 1) ? stringHex : '0' + stringHex;
	}
};